@@ -11,8 +11,8 @@ from ast import literal_eval as ale
 import numpy as np
 import pandas as pd
                     
-from npyx.utils import smooth, thresh_consec, npa
-from npyx.gl import get_units
+from npyx.utils import smooth, thresh_consec, npa, assert_int, assert_float
+from npyx.gl import get_units, assert_multi, get_dataset_id, get_ds_table
 from npyx.io import read_spikeglx_meta
 
 def ids(dp, unit, sav=True, prnt=False, subset_selection='all', again=False):
@@ -34,32 +34,36 @@ def ids(dp, unit, sav=True, prnt=False, subset_selection='all', again=False):
     assert unit in get_units(dp), 'WARNING unit {} not found in dataset {}!'.format(unit, dp)
     # Search if the variable is already saved in dp/routinesMemory
     dprm = Path(dp,'routinesMemory')
-    fn='ids{}({}).npy'.format(unit, str(subset_selection)[0:10].replace(' ', ''))
+    fn='ids{}.npy'.format(unit)
     if not op.isdir(dprm): os.makedirs(dprm)
     if op.exists(Path(dprm,fn)) and not again:
         if prnt: print("File {} found in routines memory.".format(fn))
-        return np.asarray(np.load(Path(dprm,fn)), dtype='int64')
+        indices = np.asarray(np.load(Path(dprm,fn)), dtype='int64')
     # if not, compute it
-    if prnt:
-        print("File {} not found in routines memory. Will be computed from source files.".format(fn))
-    if type(unit) in [str, np.str_]:
-        ds_i, unt = unit.split('_'); ds_i, unt = ale(ds_i), ale(unt)
-        spike_clusters_samples = np.load(Path(dp, 'merged_clusters_spikes.npy'))
-        dataset_mask=(spike_clusters_samples[:, 0]==ds_i); unit_mask=(spike_clusters_samples[:, 1]==unt)
-        indices = np.nonzero(dataset_mask&unit_mask)[0]
-        indices=np.reshape(indices, (max(indices.shape), ))
     else:
-        try:unit=int(unit)
-        except:pass
-    if type(unit) is int:
-        spike_clusters = np.load(Path(dp,"spike_clusters.npy"))
-        indices = np.nonzero(spike_clusters==unit)[0].squeeze()
-    if type(unit) not in [str, np.str_, int]:
-        print('WARNING unit {} type ({}) not handled!'.format(unit, type(unit)))
-        return
-    
+        if prnt: print(f"File {fn} not found in routines memory. Will be computed from source files.")
+        if not (assert_int(unit)|assert_float(unit)): raise TypeError(f'WARNING unit {unit} type ({type(unit)}) not handled!')
+        assert unit in get_units(dp), f'WARNING unit {unit} not found in dataset {dp}!'
+        if assert_multi(dp):
+            ds_table = get_ds_table(dp)
+            if ds_table.shape[0]>1: # If several datasets in prophyler
+                spike_clusters = np.load(Path(dp,"spike_clusters.npy"), mmap_mode='r')
+                indices = np.nonzero(spike_clusters==unit)[0].ravel()
+            else:
+                ds_i, unt = get_dataset_id(unit)
+                spike_clusters = np.load(Path(ds_table.loc['dp'][ds_i],"spike_clusters.npy"), mmap_mode='r')
+                indices = np.nonzero(spike_clusters==unt)[0].ravel()
+        else:
+            spike_clusters = np.load(Path(dp,"spike_clusters.npy"), mmap_mode='r')
+            indices = np.nonzero(spike_clusters==unit)[0].ravel()
+            
+        # Save it
+        if sav:
+            np.save(Path(dprm,fn), indices)
+        
     # Optional selection of a section of the recording.
-    if type(subset_selection) not in [str,np.str_]: # else, eq to subset_selection=[(0, spike_samples[-1])] # in samples
+    # Always computed because cannot reasonably be part of file name.
+    if subset_selection!='all': # else, eq to subset_selection=[(0, spike_samples[-1])] # in samples
         try: subset_selection[0][0]
         except: raise TypeError("ERROR subset_selection should be either a string or a list of format [(t1, t2), (t3, t4), ...]!!")
         fs=read_spikeglx_meta(dp)['sRateHz']
@@ -69,10 +73,6 @@ def ids(dp, unit, sav=True, prnt=False, subset_selection='all', again=False):
             sec_bool[(train>=section[0]*fs)&(train<=section[1]*fs)]=True # comparison in samples
         indices=indices[sec_bool]
     
-    # Save it
-    if sav:
-        np.save(Path(dprm,fn), indices)
-    # Either return or draw to global namespace
     return indices
 
     
@@ -96,7 +96,7 @@ def trn(dp, unit, sav=True, prnt=False, subset_selection='all', again=False, enf
     
     # Search if the variable is already saved in dp/routinesMemory
     dprm = Path(dp,'routinesMemory')
-    fn='trn{}({})_{}.npy'.format(unit, str(subset_selection)[0:10].replace(' ', ''), enforced_rp)
+    fn='trn{}_{}.npy'.format(unit, enforced_rp)
     if not op.isdir(dprm): os.makedirs(dprm)
     if op.exists(Path(dprm,fn)) and not again:
         if prnt: print("File {} found in routines memory.".format(fn))
@@ -104,49 +104,44 @@ def trn(dp, unit, sav=True, prnt=False, subset_selection='all', again=False, enf
 
     # if not, compute it
     else:
-        if prnt:
-            print(f"File {fn} not found in routines memory. Will be computed from source files.")
-        
+        if prnt: print(f"File {fn} not found in routines memory. Will be computed from source files.")
+        if not (assert_int(unit)|assert_float(unit)): raise TypeError(f'WARNING unit {unit} type ({type(unit)}) not handled!')
         assert unit in get_units(dp), f'WARNING unit {unit} not found in dataset {dp}!'
-        if type(unit) in [str, np.str_]:
-            ds_i, unt = unit.split('_'); ds_i, unt = ale(ds_i), ale(unt)
-            ds_table=pd.read_csv(Path(dp, 'datasets_table.csv'), index_col='dataset_i')
+        if assert_multi(dp):
+            ds_table = get_ds_table(dp)
             if ds_table.shape[0]>1: # If several datasets in prophyler
-                spike_clusters_samples = np.load(Path(dp, 'merged_clusters_spikes.npy'))
-                dataset_mask=(spike_clusters_samples[:, 0]==ds_i); unit_mask=(spike_clusters_samples[:, 1]==unt)
-                train = spike_clusters_samples[dataset_mask&unit_mask, 2].squeeze().astype(np.int64)
+                spike_clusters = np.load(Path(dp,"spike_clusters.npy"), mmap_mode='r')
+                spike_samples = np.load(Path(dp,'spike_times.npy'), mmap_mode='r')
+                train = spike_samples[spike_clusters==unit].ravel()
             else:
-                spike_clusters = np.load(Path(ds_table['dp'][0],"spike_clusters.npy"))
-                spike_samples = np.load(Path(ds_table['dp'][0],'spike_times.npy'))
-                train = spike_samples[spike_clusters==unt].squeeze()
+                ds_i, unt = get_dataset_id(unit)
+                spike_clusters = np.load(Path(ds_table['dp'][ds_i],"spike_clusters.npy"), mmap_mode='r')
+                spike_samples = np.load(Path(ds_table['dp'][ds_i],'spike_times.npy'), mmap_mode='r')
+                train = spike_samples[spike_clusters==unt].ravel()
         else:
-            try:unit=int(unit)
-            except:pass
-        if type(unit) is int:
             spike_clusters = np.load(Path(dp,"spike_clusters.npy"), mmap_mode='r')
             spike_samples = np.load(Path(dp,'spike_times.npy'), mmap_mode='r')
             train = spike_samples[spike_clusters==unit].ravel()
         
-        if type(unit) not in [str, np.str_, int]:
-            raise TypeError(f'WARNING unit {unit} type ({type(unit)}) not handled!')
-        
         # Filter out spike duplicates (spikes following an ISI shorter than enforced_rp)
         fs=read_spikeglx_meta(dp)['sRateHz']
         train=train[np.append(True, np.diff(train)>=enforced_rp*fs/1000)]
         
-        # Optional selection of a section of the recording.
-        if type(subset_selection) not in [str,np.str_]: # else, eq to subset_selection=[(0, spike_samples[-1])] # in samples
-            try: subset_selection[0][0]
-            except: raise TypeError("ERROR subset_selection should be either a string or a list of format [(t1, t2), (t3, t4), ...]!!")
-            sec_bool=np.zeros(len(train), dtype=np.bool)
-            for section in subset_selection:
-                sec_bool[(train>=section[0]*fs)&(train<=section[1]*fs)]=True # comparison in samples
-            train=train[sec_bool]
-                
         # Save it
         if sav:
             np.save(Path(dprm,fn), train)
-    # Either return or draw to global namespace
+            
+    # Optional selection of a section of the recording.
+    # Always computed because cannot reasonably be part of file name.
+    if subset_selection!='all': # else, eq to subset_selection=[(0, spike_samples[-1])] (in samples)
+        try: subset_selection[0][0]
+        except: raise TypeError("ERROR subset_selection should be either a string or a list of format [(t1, t2), (t3, t4), ...]!!")
+        fs=read_spikeglx_meta(dp)['sRateHz']
+        sec_bool=np.zeros(len(train), dtype=np.bool)
+        for section in subset_selection:
+            sec_bool[(train>=section[0]*fs)&(train<=section[1]*fs)]=True # comparison in samples
+        train=train[sec_bool]
+        
     return train
 
 def isi(dp, unit, enforced_rp=0, sav=True, prnt=False, subset_selection='all', again=False):
